/************************************************************
 **
 ** Functions for the p3d Monitor
 **
 ************************************************************/

#ifdef VXWORKS
#include <vxWorks.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <math.h>
/* for memset */
#include <string.h>

#include <time.h>
#include <sys/time.h>

#include "libp3d.h"
#include "p3dInt.h"



/*===== Local Functions =======================*/
double ngauss (double    x,
	       double    m, 
	       double    sigma );

double p3dShapeDistrib (double    dist, 
			double    maxDist,
			double    obs,  
			double    maxAngle,
			double    sigmaAngle );


/**************************************************************************************
 *
 * Calculates a normalized Gauss function
 *  ie: values from 0 to 1.0
 *
 **************************************************************************************/

double
ngauss (double    x,
	double    m,        /* m = average (moyenne ou Esperance) */
	double    sigma )    /* sigma^2 = variance  */
  
{
  /* Le coefficient affecte a la gaussienne est normalement: */
  /* (1/sqrt(2.0*M_PI*sigma*sigma)) */
  return ( exp(-(x-m)*(x-m)/(2.0*sigma*sigma)) );
}



/**************************************************************************************
 *
 * "Shaping" function for the "distance" obs-pred, for one angle
 *
 **************************************************************************************/

double
p3dShapeDistrib (double    dist,     /* dist (pred-obs) */
		 double    maxDist,  /* Max dist(pred-obs) */
		 double    obs,      /* obs value   */	      
		 double    maxAngle,   /* Max Angle authorized */
		 double    sigmaAngle)
{
  double sigmaObs, sigmaDist;    /* standard deviation for the gaussians */
  double fdist,fobs;

  /* Check */
  if( (maxAngle <= 0.0) || (maxDist <= 0.0)) {
    fprintf(stderr, "p3dMonitor shapeDistrib: Error: provided max is <= 0.0\n");
	return -1.0;
  }

  /* the sigma for the gaussian functions is set  */
  sigmaObs  = (maxAngle - 3.0*sigmaAngle)/3.0 ;
  sigmaDist = (maxDist  - 3.0*sigmaAngle)/3.0 ;

  fdist = ngauss(fabs(dist),maxDist,  sigmaDist);
  fobs  = ngauss(fabs(obs), maxAngle, sigmaObs );

  return( fdist + fobs*(1.0-fdist) );
}


